<?php

    /** @noinspection PhpMissingFieldTypeInspection */

    namespace TamerLib\Objects;

    use Exception;
    use LogLib\Log;
    use Symfony\Component\Process\PhpExecutableFinder;
    use Symfony\Component\Process\Process;
    use TamerLib\Classes\Utilities;
    use TamerLib\Exceptions\WorkerFailedException;

    class WorkerInstance
    {
        /**
         * @var WorkerConfiguration
         */
        private $configuration;

        /**
         * @var null
         */
        private $path;

        /**
         * @var Process|null
         */
        private $process;

        /**
         * WorkerInstance constructor.
         *
         * @param WorkerConfiguration $configuration
         * @param string $path
         */
        public function __construct(WorkerConfiguration $configuration, string $path)
        {
            $this->configuration = $configuration;
            $this->path = $path;
        }

        /**
         * Determines if the worker is running.
         *
         * @return bool
         */
        public function isRunning(): bool
        {
            if(is_null($this->process))
            {
                return false;
            }

            return $this->process->isRunning();
        }

        /**
         * Terminates the worker process.
         *
         * @return void
         */
        public function stop(): void
        {
            if(is_null($this->process))
            {
                return;
            }

            try
            {
                $this->process->stop();
            }
            catch(Exception $e)
            {
                Log::warning(Utilities::getName(), sprintf('Failed to stop worker %s', $this->configuration->getWorkerId()), $e);
            }
            finally
            {
                $this->process = null;
                Log::debug(Utilities::getName(), sprintf('Stopped worker %s', $this->configuration->getWorkerId()));
            }
        }

        /**
         * Starts the process for the worker.
         *
         * @return void
         * @throws WorkerFailedException
         */
        public function start(): void
        {
            if($this->isRunning())
            {
                return;
            }

            // Get the current processes arguments and pass them to the new process.
            $pass_args = $_SERVER['argv'];
            if(count($pass_args) > 1)
            {
                array_shift($pass_args);
            }

            $php_bin = (new PhpExecutableFinder())->find();
            $process = new Process(array_merge([$php_bin, $this->path], $pass_args));
            $process->setWorkingDirectory(getcwd());
            $process->setEnv($this->configuration->toEnvironment());

            try
            {
                Log::debug(Utilities::getName(), sprintf('Executing %s', $process->getCommandLine()));
                $process->start();
            }
            catch(Exception $e)
            {
                throw new WorkerFailedException(sprintf('Failed to start worker %s', $this->configuration->getWorkerId()), $e);
            }
            finally
            {
                $this->process = $process;
            }
        }

        /**
         * Restarts the worker.
         *
         * @return void
         * @throws WorkerFailedException
         */
        public function restart(): void
        {
            $this->stop();
            $this->start();
        }

        /**
         * Returns the last output from the worker.
         *
         * @return string
         */
        public function getOutput(): string
        {
            if(is_null($this->process))
            {
                return '';
            }

            return Utilities::getLatestOutput($this->process);
        }

        /**
         * @return Process|null
         */
        public function getProcess(): ?Process
        {
            return $this->process;
        }

        /**
         * @return WorkerConfiguration
         */
        public function getConfiguration(): WorkerConfiguration
        {
            return $this->configuration;
        }
    }